﻿using System;
using System.Threading.Tasks;
using ELTE.TicTacToeGame.Model;
using ELTE.TicTacToeGame.Persistence;
using ELTE.TicTacToeGame.View;
using ELTE.TicTacToeGame.ViewModel;
using Xamarin.Forms;

namespace ELTE.TicTacToeGame
{
    public class App : Application
    {
        #region Private fields

        private IPersistence _persistence;
        private TicTacToeModel _ticTacToeModel;
        private TicTacToeViewModel _ticTacToeViewModel;
        private GamePage _gamePage;

        private IStore _store;
        private StoredGameBrowserModel _storedGameBrowserModel;
        private StoredGameBrowserViewModel _storedGameBrowserViewModel;
        private LoadGamePage _loadGamePage;
        private SaveGamePage _saveGamePage;

        private NavigationPage _mainPage;

        #endregion

        #region Constructors

        /// <summary>
        /// Alkalmazás példányosítása.
        /// </summary>
        public App()
        {
            // a játék összeállítása
            _persistence = DependencyService.Get<IPersistence>(); // a perzisztencia betöltése az adott platformon

            _ticTacToeModel = new TicTacToeModel(_persistence);
            _ticTacToeModel.GameWon += new EventHandler<GameWonEventArgs>(Model_GameWon);
            _ticTacToeModel.GameOver += new EventHandler(Model_GameOver);

            _ticTacToeViewModel = new TicTacToeViewModel(_ticTacToeModel);
            _ticTacToeViewModel.LoadGame += new EventHandler(TicTacToeViewModel_LoadGame); // kezeljük a nézetmodell eseményeit
            _ticTacToeViewModel.SaveGame += new EventHandler(TicTacToeViewModel_SaveGame);

            _gamePage = new GamePage();
            _gamePage.BindingContext = _ticTacToeViewModel;

            // a játékmentések kezelésének összeállítása
            _store = DependencyService.Get<IStore>(); // a perzisztencia betöltése az adott platformon
            _storedGameBrowserModel = new StoredGameBrowserModel(_store);
            _storedGameBrowserViewModel = new StoredGameBrowserViewModel(_storedGameBrowserModel);
            _storedGameBrowserViewModel.GameLoading += new EventHandler<StoredGameEventArgs>(StoredGameBrowserViewModel_GameLoading);
            _storedGameBrowserViewModel.GameSaving += new EventHandler<StoredGameEventArgs>(StoredGameBrowserViewModel_GameSaving);

            _loadGamePage = new LoadGamePage();
            _loadGamePage.BindingContext = _storedGameBrowserViewModel;

            _saveGamePage = new SaveGamePage();
            _saveGamePage.BindingContext = _storedGameBrowserViewModel;

            // nézet beállítása
            _mainPage = new NavigationPage(_gamePage); // egy navigációs lapot hazsnálunk fel a három nézet kezelésére

            MainPage = _mainPage;
        }

        #endregion

        #region Application event methods

        protected override void OnStart()
        {
            // betöltjük a felfüggesztett játékot, amennyiben van
            try
            {
                Task.Run(async () =>
                {
                    await _ticTacToeModel.LoadGameAsync("SuspendedGame");
                    _ticTacToeViewModel.RefreshTable();
                });
            }
            catch { }
        }

        protected override void OnSleep()
        {
            // elmentjük a jelenleg folyó játékot
            try
            {
                Task.Run(async () => await _ticTacToeModel.SaveGameAsync("SuspendedGame"));
                // az aszinkron tevékenységet taszkban futtatjuk
            }
            catch { }
        }

        protected override void OnResume()
        {
            // betöltjük a felfüggesztett játékot, amennyiben van
            try
            {
                Task.Run(async () =>
                {
                    await _ticTacToeModel.LoadGameAsync("SuspendedGame");
                    _ticTacToeViewModel.RefreshTable();
                });
            }
            catch { }
        }

        #endregion

        #region Model event handlers

        /// <summary>
        /// Játék megnyerésének eseménykezelése.
        /// </summary>
        private async void Model_GameWon(object sender, GameWonEventArgs e)
        {
            switch (e.Player)
            {
                case Player.PlayerO:
                    await MainPage.DisplayAlert("Játék vége!", "A kör játékos győzött!", "OK");
                    break;
                case Player.PlayerX:
                    await MainPage.DisplayAlert("Játék vége!", "A kereszt játékos győzött!", "OK");
                    break;
            }
        }

        /// <summary>
        /// Játék végének eseménykezelése.
        /// </summary>
        private async void Model_GameOver(object sender, EventArgs e)
        {
            await MainPage.DisplayAlert("Döntetlen játék!", "Játék vége!", "OK");
        }

        #endregion

        #region ViewModel event handlers

        /// <summary>
        /// Játék betöltésének eseménykezelője.
        /// </summary>
        private async void TicTacToeViewModel_LoadGame(object sender, System.EventArgs e)
        {
            await _storedGameBrowserModel.UpdateAsync(); // frissítjük a tárolt játékok listáját
            await _mainPage.PushAsync(_loadGamePage); // átnavigálunk a lapra
            _ticTacToeViewModel.RefreshTable();
        }

        /// <summary>
        /// Játék mentésének eseménykezelője.
        /// </summary>
        private async void TicTacToeViewModel_SaveGame(object sender, System.EventArgs e)
        {
            await _storedGameBrowserModel.UpdateAsync(); // frissítjük a tárolt játékok listáját
            await _mainPage.PushAsync(_saveGamePage); // átnavigálunk a lapra
        }

        /// <summary>
        /// Betöltés végrehajtásának eseménykezelője.
        /// </summary>
        private async void StoredGameBrowserViewModel_GameLoading(object sender, StoredGameEventArgs e)
        {
            await _mainPage.PopAsync(); // visszanavigálunk

            try
            {
                await _ticTacToeModel.LoadGameAsync(e.Name); // betöltjük a megadott játékot
                _ticTacToeViewModel.RefreshTable();
            }
            catch { }
        }

        /// <summary>
        /// Mentés végrehajtásának eseménykezelője.
        /// </summary>
        private async void StoredGameBrowserViewModel_GameSaving(object sender, StoredGameEventArgs e)
        {
            await _mainPage.PopAsync(); // visszanavigálunk

            try
            {
                await _ticTacToeModel.SaveGameAsync(e.Name); // lementjük a megadott játékot
                await _storedGameBrowserModel.UpdateAsync();  // frissítjük a tárolt játékok listáját
            }
            catch
            {
                await MainPage.DisplayAlert("Hiba!", "A mentés sikertelen volt.", "OK");
            }
        }

        #endregion
    }
}
